'use strict';

/* exported logger */

var logger = require('./logger');

var OriginalServerConnector = require('../../main/js/ServerConnector');

var fs = require('fs');
var request = require('request');

var ServerConnectorMock = OriginalServerConnector;

ServerConnectorMock.init = function() {
  this._customMap = null;
  this._sessionData = undefined;
  
  // add listener types
};
ServerConnectorMock.init();


ServerConnectorMock.getMaxOverlayColorInt = function() {
  return 0xFF0000;
};

ServerConnectorMock.getMinOverlayColorInt = function() {
  return 0x00FF00;
};

ServerConnectorMock.readFile = function(url) {
  return new Promise(function(resolve, reject) {
    if (url.indexOf("http") === 0) {
      request.get(url, function(error, response, body) {
        if (error) {
          reject(error);
        } else if (response.statusCode !== 200) {
          reject(response);
        } else {
          resolve(body);
        }
      });
    } else {
      fs.readFile(url, 'utf8', function(err, content) {
        if (err) {
          reject(err);
        } else {
          resolve(content);
        }
      });
    }
  });
};

ServerConnectorMock.sendPostRequest = function(url, params) {
  var self = this;
  return new Promise(function(resolve, reject) {
    if (url.indexOf("http") === 0) {
      request.post({url:url, form:params}, function(error, response, body) {
        if (error) {
          reject(error);

        } else if (response.statusCode !== 200) {
          reject(response);
        } else {
          resolve(body);
        }
      });
    } else {
      var mockUrl = url+"/"+self.createGetParams(params); 
      fs.readFile(mockUrl, 'utf8', function(err, content) {
        if (err) {
          reject(err);
        } else {
          resolve(content);
        }
      });
    }
  });
};

ServerConnectorMock.getApiBaseUrl = function() {
  return "./testFiles/apiCalls/";
};

var originalGetApiUrl = OriginalServerConnector.getApiUrl;

ServerConnectorMock.getApiUrl = function(paramObj) {
  //replace '?' with '/'
  //the call is done on ServerConnectorObject (so 'this' is set properly)
  return originalGetApiUrl.call(this, paramObj).replace(/\?/g, '/');
};

module.exports = ServerConnectorMock;
