"use strict";

var Helper = require('../helper');

require("../mocha-config.js");

var MiRna = require('../../../main/js/map/data/MiRna');
var MiRnaPanel = require('../../../main/js/gui/MiRnaPanel');


var chai = require('chai');
var assert = chai.assert;
var logger = require('../logger');

describe('MiRnaPanel', function() {

  var helper;
  before(function() {
    helper = new Helper();
  });

  it('contructor', function() {
    var div = helper.createMiRnaTab();

    var map = helper.createCustomMap();
    helper.createMiRnaDbOverlay(map);

    new MiRnaPanel({
      element : div,
      customMap : map
    });
    assert.equal(logger.getWarnings().length, 0);
  });

  it('createMiRnaeader for empty', function() {
    var div = helper.createMiRnaTab();

    var map = helper.createCustomMap();
    helper.createMiRnaDbOverlay(map);

    var panel = new MiRnaPanel({
      element : div,
      customMap : map
    });

    assert.ok(panel.createPreamble().innerHTML.indexOf("NOT FOUND") > 0);
  });

  it('createMiRnaeader for empty', function() {
    var div = helper.createMiRnaTab();

    var map = helper.createCustomMap();
    helper.createMiRnaDbOverlay(map);

    var panel = new MiRnaPanel({
      element : div,
      customMap : map
    });

    assert.ok(panel.createPreamble(new MiRna()).innerHTML.indexOf("NOT FOUND") > 0);
  });

  it('on searchResults changed', function() {
    var div = helper.createMiRnaTab();
    var map = helper.createCustomMap();
    map.getModel().setId(15781);
    var miRnaDbOverlay = helper.createMiRnaDbOverlay(map);

    new MiRnaPanel({
      element : div,
      customMap : map
    });

    return miRnaDbOverlay.searchByQuery("hsa-miR-125a-3p").then(function() {
      assert.equal(logger.getWarnings().length, 0);
      assert.ok(div.innerHTML.indexOf("hsa-miR-125a-3p") >= 0);
    });
  });

  it('searchByQuery', function() {
    var div = helper.createMiRnaTab();
    var map = helper.createCustomMap();
    map.getModel().setId(15781);
    helper.createMiRnaDbOverlay(map);

    var panel = new MiRnaPanel({
      element : div,
      customMap : map
    });

    panel.getSearchInput().value = "hsa-miR-125a-3p";

    return panel.searchByQuery().then(function() {
      assert.equal(logger.getWarnings().length, 0);
      assert.ok(div.innerHTML.indexOf("http://www.mirbase.org/cgi-bin/mirna_entry.pl?acc") >= 0);
    });
  });

});
