"use strict";

require("./mocha-config");

var Alias = require("../../main/js/map/data/Alias");
var AbstractCustomMap = require("../../main/js/map/AbstractCustomMap");
var ChemicalDbOverlay = require("../../main/js/map/overlay/ChemicalDbOverlay");
var Comment = require("../../main/js/map/data/Comment");
var CommentDbOverlay = require("../../main/js/map/overlay/CommentDbOverlay");
var CustomMap = require("../../main/js/map/CustomMap");
var CustomMapOptions = require("../../main/js/map/CustomMapOptions");
var DrugDbOverlay = require("../../main/js/map/overlay/DrugDbOverlay");
var IdentifiedElement = require("../../main/js/map/data/IdentifiedElement");
var LayoutAlias = require("../../main/js/map/data/LayoutAlias");
var LayoutData = require("../../main/js/map/data/LayoutData");
var LayoutReaction = require("../../main/js/map/data/LayoutReaction");
var MiRnaDbOverlay = require("../../main/js/map/overlay/MiRnaDbOverlay");
var Model = require("../../main/js/map/data/MapModel");
var AbstractDbOverlay = require("../../main/js/map/overlay/AbstractDbOverlay");
var Project = require("../../main/js/map/data/Project");
var Reaction = require("../../main/js/map/data/Reaction");
var SearchDbOverlay = require("../../main/js/map/overlay/SearchDbOverlay");

function Helper() {
  this.idCounter = 1000000;
}

Helper.prototype.createSearchTab = function() {
  return this.createPanelTab("searchTab");
};

Helper.prototype.createDrugTab = function() {
  return this.createPanelTab("drugTab");
};

Helper.prototype.createChemicalTab = function() {
  return this.createPanelTab("chemicalTab");
};

Helper.prototype.createMiRnaTab = function() {
  return this.createPanelTab("mirnaTab");
};

Helper.prototype.createPanelTab = function(id) {
  var result = document.createElement("div");
  result.id = id;
  var searchQueryDiv = document.createElement("div");
  searchQueryDiv.setAttribute("name", "searchQuery");
  result.appendChild(searchQueryDiv);

  var searchButton = document.createElement("img");
  searchButton.setAttribute("name", "searchButton");
  searchQueryDiv.appendChild(searchButton);

  var searchInput = document.createElement("input");
  searchInput.setAttribute("name", "searchInput");
  searchQueryDiv.appendChild(searchInput);

  var searchPerfectMatch = document.createElement("input");
  searchPerfectMatch.type = "checkbox";
  searchPerfectMatch.setAttribute("name", "searchPerfectMatch");
  searchQueryDiv.appendChild(searchPerfectMatch);

  var searchResultsDiv = document.createElement("div");
  searchResultsDiv.setAttribute("name", "searchResults");
  result.appendChild(searchResultsDiv);

  var navDiv = document.createElement("ul");
  navDiv.className = "nav nav-tabs";
  navDiv.innerHTML = '<li class="active"><a href="#set1"/></li>';

  var contentDiv = document.createElement("div");
  contentDiv.className = "tab-content";
  contentDiv.innerHTML = '<div class="tab-pane fade active in" id="set1"/>';

  searchResultsDiv.appendChild(navDiv);
  searchResultsDiv.appendChild(contentDiv);

  return result;
};

Helper.prototype.createOverlayTab = function() {
  var result = document.createElement("div");
  result.id = "overlayTab";
  var generalOverlaysDiv = document.createElement("div");
  generalOverlaysDiv.setAttribute("name", "generalOverlays");
  result.appendChild(generalOverlaysDiv);
  
  var generalOverlaysTabDiv = document.createElement("table");
  generalOverlaysTabDiv.setAttribute("name", "generalOverlaysTab");
  generalOverlaysDiv.appendChild(generalOverlaysTabDiv);
  
  var customOverlaysDiv = document.createElement("div");
  customOverlaysDiv.setAttribute("name", "customOverlays");
  result.appendChild(customOverlaysDiv);
  
  var customOverlaysTabDiv = document.createElement("table");
  customOverlaysTabDiv.setAttribute("name", "customOverlaysTab");
  customOverlaysDiv.appendChild(customOverlaysTabDiv);
  
  return result;
};

Helper.prototype.createCommentDbOverlay = function(map) {
  var result = new CommentDbOverlay({
    map : map,
    name : "comment",
    allowGeneralSearch : true
  });
  map.registerSource(result);
  return result;
};

Helper.prototype.createSearchDbOverlay = function(map) {
  var result = new SearchDbOverlay({
    map : map,
    name : "search",
  });
  map.registerSource(result);
  return result;
};

Helper.prototype.createDbOverlay = function(map) {
  var result = new AbstractDbOverlay({
    map : map,
    name : "search",
  });
  map.registerSource(result);
  return result;
};

Helper.prototype.createDrugDbOverlay = function(map) {
  var result = new DrugDbOverlay({
    map : map,
    name : "drug",
    allowGeneralSearch : true,
    allowSearchById : true,
  });
  map.registerSource(result);
  return result;
};

Helper.prototype.createChemicalDbOverlay = function(map) {
  var result = new ChemicalDbOverlay({
    map : map,
    name : "chemical",
    allowGeneralSearch : true,
    allowSearchById : true,
  });
  map.registerSource(result);
  return result;
};

Helper.prototype.createMiRnaDbOverlay = function(map) {
  var result = new MiRnaDbOverlay({
    map : map,
    name : "mirna",
    allowGeneralSearch : true,
    allowSearchById : true,
  });
  map.registerSource(result);
  return result;
};

Helper.prototype.createComment = function(element) {
  var elementType = "ALIAS";
  var elementId = this.idCounter++;
  var id = this.idCounter++;
  var modelId = this.idCounter++;

  if (element !== undefined) {
    if (element instanceof Alias) {
      elementType = "ALIAS";
      elementId = element.getId();
      modelId = element.getModelId();
    } else {
      throw new Error("Unknown element type: " + element);
    }
  }

  var result = new Comment({
    elementId : elementId,
    type : elementType,
    modelId : modelId,
    icon : "icons/comment.png",
    id : id,
    pinned : true,
    coord : {
      x : 321.5,
      y : 289.0
    },
    removed : false,
    title : "title fo comment: " + id,
    content : "content of the comment # " + id
  });

  return result;
};

Helper.prototype.createProject = function() {
  var result = new Project();
  result.setProjectId("testId");
  result.setId(this.idCounter++);
  result.setModel(this.createModel());
  return result;
};

Helper.prototype.createAlias = function(map) {
  var mapId;
  if (map === undefined) {
    mapId = this.idCounter++;
  } else {
    mapId = map.getId();
  }
  var result = new Alias({
    idObject : this.idCounter++,
    name : "Test element",
    type : "RNA",
    modelId : mapId,
    bounds : {
      x : 10.0,
      y : 20.0,
      width : 30.0,
      height : 40.0,
    },
  });
  if (map !== undefined) {
    map.getModel().addAlias(result);
  }
  return result;
};

Helper.prototype.createLayoutAlias = function(alias) {
  var id;
  if (alias instanceof Alias) {
    id = alias.getId();
  } else {
    id = this.idCounter++;
  }
  var result = new LayoutAlias({
    idObject : id,
    value : 0.2,
    color : {
      a : 23
    },
    geneVariations : [ {} ]
  });
  return result;
};

Helper.prototype.createLayoutReaction = function(reaction) {
  var reactionId;
  if (reaction instanceof Reaction) {
    reactionId = reaction.getId();
  } else {
    reactionId = this.idCounter++;
  }

  var result = new LayoutReaction({
    idObject : reactionId,
    width : 3.4,
    color : {
      a : 24,
    },
    reverse : true
  });
  return result;
};

Helper.prototype.createIdentifiedElement = function(element) {
  if (element === undefined) {
    return new IdentifiedElement({
      type : "ALIAS",
      id : this.idCounter++,
      modelId : this.idCounter++,
    });
  }
  return new IdentifiedElement(element);
};

Helper.prototype.createReaction = function(map) {
  var mapId = null;
  if (map !== undefined) {
    mapId = map.getId();
  } else {
    mapId = this.idCounter++;
  }
  var result = new Reaction({
    idObject : this.idCounter++,
    lines : [ {
      start : {
        x : 434.8904109589041,
        y : 85.0
      },
      end : {
        x : 410.8341500923087,
        y : 104.95576185524392
      },
      type : "START"
    }, {
      start : {
        x : 404.6769250286157,
        y : 110.06345991944379
      },
      end : {
        x : 380.62066416202026,
        y : 130.0192217746877
      },
      type : "END"
    } ],
    centerPoint : new google.maps.Point(0, 0),
    modelId : mapId,
  });
  return result;
};

Helper.prototype.createOptions = function(project) {
  if (project === undefined) {
    project = this.createProject();
  }

  var options = {
    mapDiv : testDiv,
    project : project,
    dataCollections : [ {
      name : "search",
    } ],
  };
  return options;
};

Helper.prototype.createModel = function() {
  var result = new Model();
  result.setId(this.idCounter++);
  result.setTileSize(256);
  result.setWidth(26547.3333333333);
  result.setHeight(800);
  result.setMaxZoom(8);
  result.setMinZoom(2);
  result.setCenterLatLng({
    lat : 10,
    lng : 20
  });
  result.setTopLeftLatLng({
    lat : 8,
    lng : 8
  });
  result.setBottomRightLatLng({
    lat : 30,
    lng : 30
  });

  var layout = this.createLayout();
  result.addLayout(layout);
  return result;
};

Helper.prototype.createLayout = function() {
  var id = this.idCounter++;
  var layout = new LayoutData({
    idObject : id,
    name : "testLayout" + id,
  });
  return layout;
};

Helper.prototype.createGoogleMap = function() {
  var result = new google.maps.Map(testDiv, {
    center : {
      lat : -34.397,
      lng : 150.644
    },
    scrollwheel : false,
    zoom : 8
  });
  return result;
};

Helper.prototype.createCustomMapOptions = function() {
  var project = this.createProject();

  var result = new CustomMapOptions({
    markerOptimization : true,
    project : project,
    mapDiv : testDiv
  });

  return result;
};

Helper.prototype.createAbstractCustomMap = function() {
  var options = this.createCustomMapOptions();
  var result = new AbstractCustomMap(options.getProject().getModel(), options);

  return result;
};

Helper.prototype.createCustomMap = function() {
  var options = this.createCustomMapOptions();
  var result = new CustomMap(options);
  return result;
};

module.exports = Helper;
